from qtstrap import QColor


class colors:
    navy = '#001f3f'
    blue = '#0074D9'
    aqua = '#7FDBFF'
    teal = '#39CCCC'
    olive = '#3D9970'
    green = '#2ECC40'
    lime = '#01FF70'
    yellow = '#FFDC00'
    orange = '#FF851B'
    red = '#FF4136'
    maroon = '#85144b'
    fuchsia = '#F012BE'
    purple = '#B10DC9'
    black = '#111111'
    gray = '#AAAAAA'
    silver = '#DDDDDD'
    white = '#FFFFFF'


class qcolors:
    navy = QColor('#001f3f')
    blue = QColor('#0074D9')
    aqua = QColor('#7FDBFF')
    teal = QColor('#39CCCC')
    olive = QColor('#3D9970')
    green = QColor('#2ECC40')
    lime = QColor('#01FF70')
    yellow = QColor('#FFDC00')
    orange = QColor('#FF851B')
    red = QColor('#FF4136')
    maroon = QColor('#85144b')
    fuchsia = QColor('#F012BE')
    purple = QColor('#B10DC9')
    black = QColor('#111111')
    gray = QColor('#AAAAAA')
    silver = QColor('#DDDDDD')
    white = QColor('#FFFFFF')